/*
 * Copyright (c) 2024 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/kernel.h>
#include <zephyr/logging/log.h>

/* STEP 1.2 - Include the header files for SPI, GPIO and devicetree */
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/spi.h>

#include "ad5940.h"

LOG_MODULE_REGISTER(Lesson5_Exercise1, LOG_LEVEL_INF);
#define DELAY_VALUES 1000

const struct gpio_dt_spec ledspec = GPIO_DT_SPEC_GET(DT_NODELABEL(led0), gpios);

/* STEP 3 - Retrieve the API-device structure */
// #define SPIOP	SPI_WORD_SET(8) | SPI_TRANSFER_MSB
// struct spi_dt_spec spispec = SPI_DT_SPEC_GET(DT_NODELABEL(bme280), SPIOP, 0);

const struct device *spi_dev;
const struct device *gpio_dev= DEVICE_DT_GET(DT_NODELABEL(gpio0));

struct spi_config spi_cfg = {
	.operation = SPI_WORD_SET(8) | SPI_TRANSFER_MSB | SPI_MODE_CPOL | SPI_MODE_CPHA,
	.frequency = 1000000,
	.slave = 0,
};

int main(void)
{
	int err;

	err = gpio_is_ready_dt(&ledspec);
	if (!err)
	{
		LOG_ERR("Error: GPIO device is not ready, err: %d", err);
		return 0;
	}

	spi_dev = DEVICE_DT_GET(DT_NODELABEL(spi1));
	/* STEP 10.1 - Check if SPI and GPIO devices are ready */
	// err = spi_is_ready_dt(&spispec);
	// err = spi_is_ready(spi_dev);
	// if (!err) {
	if (spi_dev == NULL)
	{
		LOG_ERR("Error: SPI device is not ready, err: %d", err);
		return 0;
	}
	// printk("SPI : %s",spispec.bus->name);
	printk("SPI : %s", spi_dev->name);

	gpio_pin_configure_dt(&ledspec, GPIO_OUTPUT_ACTIVE);
	gpio_pin_configure(gpio_dev,11,GPIO_OUTPUT_INACTIVE);

	// LOG_INF("Continuously read sensor samples, compensate, and display");

	while (1)
	{
		/* STEP 10.4 - Continuously read sensor samples and toggle led */
		AD5940_Initialize();
		gpio_pin_toggle_dt(&ledspec);
		k_msleep(DELAY_VALUES);
	}

	return 0;
}

void AD5940_ReadWriteNBytes(unsigned char *pSendBuffer, unsigned char *pRecvBuff, unsigned long length)
{
	int err;
	struct spi_buf tx_spi_buf = {.buf = (void *)&pSendBuffer, .len = length};
	struct spi_buf_set tx_spi_buf_set = {.buffers = &tx_spi_buf, .count = 1};
	struct spi_buf rx_spi_bufs = {.buf = pRecvBuff, .len = length};
	struct spi_buf_set rx_spi_buf_set = {.buffers = &rx_spi_bufs, .count = 1};

	/* STEP 4.2 - Call the transceive function */
	// err = spi_transceive_dt(&spispec, &tx_spi_buf_set, &rx_spi_buf_set);
	err = spi_transceive(spi_dev, &spi_cfg, &tx_spi_buf_set, &rx_spi_buf_set);
	if (err < 0)
	{
		LOG_ERR("spi_transceive_dt() failed, err: %d", err);
	}
}

void AD5940_CsClr(void)
{
	// gpio_pin_set_dt(spispec.config.cs.gpio.pin,1);
	gpio_pin_set(gpio_dev,11,1);
}

void AD5940_CsSet(void)
{
	// gpio_pin_set_dt(spispec.config.cs.gpio.pin,0);
	gpio_pin_set(gpio_dev,11,0);
}

void AD5940_RstSet(void)
{
}

void AD5940_RstClr(void)
{
}

void AD5940_Delay10us(uint32_t time)
{
	if (time > 0)
	{
		k_usleep(time * 10); // Delay in microseconds
	}
}